@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-import socket,time,re,os,sys,traceback,threading,urllib
+import socket,time,re,os,sys,traceback,threading,urllib,cgi
 from . Events import Events
 from . common import *
 from . import config
@@ -9,6 +9,8 @@ from threading import local
 from .utill import filetype
 from kcweb.utill.cache import cache as kcwcache
 from wsgiref.simple_server import make_server
+from kcweb.utill.db import mysql as kcwmysql
+from kcweb.utill.db import sqlite as kcwsqlite
 class web:
     __name=None
     __appname=None
@@ -18,36 +20,69 @@ class web:
         self.__appname=appname
         if self.__name != '__main__':
             def apps(env, start_response):
-                try:
-                    env['BODY_DATA']=str(env['wsgi.input'].next(), encoding = "utf-8")
-                except:
-                    env['BODY_DATA']=""
                 p=(config.app['staticpath']+env['RAW_URI'].replace(' ',''))
                 status='200 ok'
                 if os.path.isfile(p):
-                    kind = filetype.guess(p)
-                    if kind is None:
-                        f=open(p,"rb")
-                        body=f.read()
-                        f.close()
-                        resheader=[
-                            ("Cache-Control","public, max-age=43200"),
-                        ]
+                    resheader=[("Cache-Control","public, max-age=2592000"),]
+                    ETag=md5(p)
+                    try:
+                        HTTP_IF_NONE_MATCH=env['HTTP_IF_NONE_MATCH']
+                    except KeyError:
+                        HTTP_IF_NONE_MATCH=None
+                    if HTTP_IF_NONE_MATCH:
+                        status="304 Not Modified"
+                        body=''
                     else:
-                        f=open(p,"rb")
-                        body=f.read()
-                        f.close()
-                        resheader=[
-                            ("Content-Type",kind.mime),
-                            ("Cache-Control","public, max-age=43200"),
-                            ("Accept-Ranges","bytes"),
-                        ]
+                        kind = filetype.guess(p)
+                        if kind is None:
+                            f=open(p,"rb")
+                            body=f.read()
+                            f.close()
+                        else:
+                            f=open(p,"rb")
+                            body=f.read()
+                            f.close()
+                            resheader=[
+                                ("Content-Type",kind.mime),
+                                ("Cache-Control","public, max-age=43200"),
+                                ("Accept-Ranges","bytes"),
+                            ]
+                            resheader.append(('Content-Type',kind.mime))
+                            resheader.append(("Accept-Ranges","bytes"))
+                    dateArray = datetime.utcfromtimestamp(times()-86400)
+                    otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
+                    resheader.append(("Last-Modified",otherStyleTime))
+                    resheader.append(("ETag",ETag))
                 else:
+                    files=None
+                    try:
+                        CONTENT_TYPE=env['CONTENT_TYPE']
+                    except:
+                        pass
+                    else:
+                        if 'multipart/form-data; boundary' in CONTENT_TYPE:#上传文件
+                            files = cgi.FieldStorage(fp=env['wsgi.input'], environ=env, keep_blank_values=True)
+                        else:
+                            files=None
+                    env['files']=files
+                    try:
+                        env['BODY_DATA']=str(env['wsgi.input'].next(), encoding = "utf-8")
+                    except:
+                        env['BODY_DATA']=""
+                    try:
+                        env['HTTP_X_REAL_IP']=env['HTTP_X_REAL_IP']
+                    except:
+                        env['HTTP_X_REAL_IP']=""
+                    try:
+                        env['url_scheme']=env['HTTP_X_REAL_IP']
+                    except:
+                        env['url_scheme']=env['wsgi.url_scheme'],
                     status,resheader,body=self.__routes(self,env)
                     if type(body) is bytes:
                         pass
                     else:
                         body=bytes(body, encoding='utf-8')
+                # print(env)
                 start_response(status,resheader)
                 return [body]
             return apps
@@ -79,7 +114,7 @@ class web:
                     port=port,
                     filename=filename
                 )
-        else:
+        elif len(sys.argv)==3 and sys.argv[2]=='--cli':
             try:
                 RAW_URI=sys.argv[1]
             except:pass
@@ -97,6 +132,7 @@ class web:
                     'HTTP_HOST':'',
                     'HTTP_COOKIE':'',
                     'REMOTE_ADDR':'',
+                    'HTTP_X_REAL_IP':'',
                     'HTTP_USER_AGENT':'',
                     'BODY_DATA':''
                 }
@@ -133,19 +169,7 @@ class web:
                 )
         except KeyboardInterrupt:
             pass
-    def __get_modular(self,header):
-        "获取模块"
-        modular=''
-        route=self.__config.route
-        if route['modular']:
-            if isinstance(route['modular'],str):
-                modular=route['modular']
-            else:
-                HTTP_HOST=header['HTTP_HOST'].split(".")[0]
-                for mk in route['modular']:
-                    if HTTP_HOST in mk:
-                        modular=mk[HTTP_HOST]
-        return modular
+    
     def __getconfigroute(self,PATH_INFO,header):
         "使用配置路由"
         route=self.__config.route
@@ -183,14 +207,44 @@ class web:
                     break
         except:pass
         return routedefault,PATH_INFO
+    def __get_modular(self,header):
+        "获取模块"
+        modular=''
+        route=self.__config.route
+        if route['modular']:
+            if isinstance(route['modular'],str):
+                modular=route['modular']
+            else:
+                HTTP_HOST=header['HTTP_HOST'].split(".")[0]
+                for mk in route['modular']:
+                    if HTTP_HOST in mk:
+                        modular=mk[HTTP_HOST]
+        return modular
+    def __get_plug(self,header):
+        "获取插件"
+        plug=''
+        route=self.__config.route
+        if route['plug']:
+            if isinstance(route['plug'],str):
+                plug=route['plug']
+            else:
+                HTTP_HOST=header['HTTP_HOST'].split(".")[0]
+                for mk in route['plug']:
+                    if HTTP_HOST in mk:
+                        plug=mk[HTTP_HOST]
+        return plug
     def defaultroute(self,header,PATH_INFO):
         "路由匹配"
         route=self.__config.route
         modular=web.__get_modular(self,header)
+        if route['plug']:
+            getplug=web.__get_plug(self,header)
+        else:
+            getplug=''
         routedefault=route['default']
         methods=route['methods']
         if routedefault:
-            plug='index'
+            plug=route['defplug']
             files=route['files']
             funct=route['funct']
         else:
@@ -204,8 +258,10 @@ class web:
         ##默认路由start #################################################################################
         
         if modular:
-            if route['plug']:  #匹配模块并且匹配了插件
-                plug=route['plug']
+            # if route['plug']:  #匹配模块并且匹配了插件
+            #     plug=route['plug']
+            if getplug:
+                plug=getplug
                 routedefault,PATH_INFO=web.__getconfigroute(
                     self,
                     PATH_INFO,
@@ -243,8 +299,10 @@ class web:
                             else:
                                 param.append(urllib.parse.unquote(path))
                         i+=1
-        elif route['plug']:  #配置版本的但没有匹配插件
-            plug=route['plug']
+        # elif route['plug']:  #配置模块但没有匹配插件
+        #     plug=route['plug']
+        elif getplug:
+            plug=getplug
             routedefault,PATH_INFO=web.__getconfigroute(
                 self,
                 PATH_INFO,
@@ -283,6 +341,9 @@ class web:
                             param.append(urllib.parse.unquote(path))
                     i+=1
         #默认路由end ############################################################
+        if not modular:
+            modular=route['defmodular']
+        
         return methods,modular,plug,files,funct,tuple(param)
     def __tran(self,data,status,resheader):
         "转换控制器返回的内容"
@@ -306,6 +367,8 @@ class web:
             body=data
         return body,status,resheader
     def __set_globals(self,header):
+        globals.HEADER.GET=header
+        # globals.HEADER.url_scheme=header['url_scheme']
         globals.HEADER.Method=header['REQUEST_METHOD']
         globals.HEADER.URL=header['RAW_URI']
         globals.HEADER.PATH_INFO=header['PATH_INFO']
@@ -313,6 +376,12 @@ class web:
         globals.HEADER.SERVER_PROTOCOL=header['SERVER_PROTOCOL']
         globals.HEADER.HTTP_HOST=header['HTTP_HOST']
         globals.HEADER.BODY_DATA=header['BODY_DATA']
+        globals.HEADER.REMOTE_ADDR=header['REMOTE_ADDR']
+        globals.HEADER.HTTP_X_REAL_IP=header['HTTP_X_REAL_IP']
+        try:
+            globals.HEADER.files=header['files']
+        except:
+            globals.HEADER.files=None
         try:
             globals.HEADER.HTTP_COOKIE=header['HTTP_COOKIE']
         except:
@@ -507,7 +576,7 @@ class web:
         
         if self.__config.app['after_request']:  #请求后执行的函数
             try:
-                data=getattr(obj,self.__config.app['after_request'])()
+                data=getattr(obj,self.__config.app['after_request'])(body,status,resheader)
                 if data:
                     body,status,resheader=web.__tran(self,data,status,resheader)
             except (AttributeError,UnboundLocalError):pass
@@ -545,6 +614,14 @@ class web:
         for key in resheader:
             resheaders.append((key,resheader[key]))
         web.__del_globals()
+
+        #关闭数据库连接
+        dbs=kcwmysql.mysql()
+        dbs.close()
+        # dbs=kcwsqlite.sqlite()
+        # dbs.close()
+
+
         if isinstance(resheaders,list):
             if not body:
                 body=''
@@ -566,19 +643,19 @@ class web:
     def __application(self,env, start_response):
         # print(env)
         try:
-            request_body_size = int(env.get('CONTENT_LENGTH', 0))
+            request_body_size = int(env.get('CONTENT_LENGTH', '0'))
         except (ValueError):
             request_body_size = 0
-        BODY_DATA=env['wsgi.input'].read(request_body_size)
-        if 'multipart/form-data; boundary' in env['CONTENT_TYPE']:
-            BODY_DATA=BODY_DATA.decode("ISO-8859-1")
-            env['BODY_DATA']=BODY_DATA
-            # lists=BODY_DATA.split("\r\n")
-            # BODY_DATA=re.sub(lists[0],'',BODY_DATA)
-            # BODY_DATA=re.sub(lists[1],'',BODY_DATA)
-            # BODY_DATA=re.sub(lists[2],'',BODY_DATA)
-            # BODY_DATA=re.sub(lists[len(lists)-2],'',BODY_DATA)
+        if 'multipart/form-data; boundary' in env['CONTENT_TYPE']:#上传文件
+            files = cgi.FieldStorage(fp=env['wsgi.input'], environ=env, keep_blank_values=True)
+            # open(form["file"].filename, 'wb').write(form["file"].value) 
+            # print(files["file"].name)
+            # print(files["name"].filename)
+            # print(files["name"].value)
+            env['BODY_DATA']=None
         else:
+            files=None
+            BODY_DATA=env['wsgi.input'].read(request_body_size)
             env['BODY_DATA']=BODY_DATA.decode()
         env['RAW_URI']=env['PATH_INFO']
         if env.get('QUERY_STRING'):
@@ -587,20 +664,25 @@ class web:
             env['REMOTE_ADDR']
         except:
             env['REMOTE_ADDR']=''
+        try:
+            env['wsgi.url_scheme']
+        except:
+            env['wsgi.url_scheme']=''
+        try:
+            env['HTTP_X_REAL_IP']
+        except:
+            env['HTTP_X_REAL_IP']=''
         try:
             env['HTTP_COOKIE']
         except:
             env['HTTP_COOKIE']=''
-        try:
-            env['REMOTE_ADDR']
-        except:
-            env['REMOTE_ADDR']=''
         try:
             env['HTTP_USER_AGENT']
         except:
             env['HTTP_USER_AGENT']=''
         reqheader={ 
             'REQUEST_METHOD':env['REQUEST_METHOD'],
+            'url_scheme':env['wsgi.url_scheme'],
             'RAW_URI':env['RAW_URI'],
             'PATH_INFO':env['PATH_INFO'],
             'QUERY_STRING':env['QUERY_STRING'],
@@ -609,28 +691,40 @@ class web:
             'HTTP_COOKIE':env['HTTP_COOKIE'],
             'REMOTE_ADDR':env['REMOTE_ADDR'],
             'HTTP_USER_AGENT':env['HTTP_USER_AGENT'],
-            'BODY_DATA':env['BODY_DATA']
+            'BODY_DATA':env['BODY_DATA'],
+            'HTTP_X_REAL_IP':env['HTTP_X_REAL_IP'],
+            'files':files
         }
         p=(config.app['staticpath']+reqheader['RAW_URI'].replace(' ',''))
         status='200 ok'
         if os.path.isfile(p):
-            kind = filetype.guess(p)
-            if kind is None:
-                f=open(p,"rb")
-                body=f.read()
-                f.close()
-                resheader=[
-                    ("Cache-Control","public, max-age=43200"),
-                ]
+            resheader=[("Cache-Control","public, max-age=2592000"),("Accept-Ranges","bytes")]
+            ETag=md5(p)
+            try:
+                HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
+            except:
+                HTTP_IF_NONE_MATCH=None
+            if HTTP_IF_NONE_MATCH:
+                status="304 Not Modified"
             else:
-                f=open(p,"rb")
-                body=f.read()
-                f.close()
-                resheader=[
-                    ("Content-Type",kind.mime),
-                    ("Cache-Control","public, max-age=43200"),
-                    ("Accept-Ranges","bytes"),
-                ]
+                kind = filetype.guess(p)
+                if kind is None:
+                    f=open(p,"rb")
+                    body=f.read()
+                    f.close()
+                    if '.css' in p[-5:]:
+                        resheader.append(("Content-Type","text/css"))
+                    elif '.js' in p[-5:]:
+                        resheader.append(("Content-Type","application/javascript"))
+                else:
+                    f=open(p,"rb")
+                    body=f.read()
+                    f.close()
+                    resheader.append(("Content-Type",kind.mime))
+            dateArray = datetime.utcfromtimestamp(times()-86400)
+            otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
+            resheader.append(("Last-Modified",otherStyleTime))
+            resheader.append(("ETag",ETag))
         else:
             status,resheader,body=web.__routes(self,reqheader)
             resheader.append(('Server', "kcweb"+config.kcweb['version']))
@@ -717,6 +811,7 @@ class web:
                 'HTTP_HOST':HTTP_HOST,
                 'HTTP_COOKIE':HTTP_COOKIE,
                 'REMOTE_ADDR':REMOTE_ADDR,
+                'HTTP_X_REAL_IP':'',
                 'HTTP_USER_AGENT':HTTP_USER_AGENT,
                 'BODY_DATA':BODY_DATA
             }
@@ -834,6 +929,7 @@ class web:
                             'HTTP_HOST':HTTP_HOST,
                             'HTTP_COOKIE':HTTP_COOKIE,
                             'REMOTE_ADDR':REMOTE_ADDR,
+                            'HTTP_X_REAL_IP':'',
                             'HTTP_USER_AGENT':HTTP_USER_AGENT,
                             'BODY_DATA':BODY_DATA
                         }
