@@ -1,7 +1,7 @@
 
-# python setup.py sdist upload
+# python3 setup.py sdist upload
 # twine upload --repository-url https://test.pypi.org/legacy/ dist/* #上传到测试
-# pip install --index-url https://test.pypi.org/simple/ kcweb   #安装测试服务上的kcweb
+# pip install --index-url https://pypi.org/simple/ kcweb   #安装测试服务上的kcweb pip3 install kcweb==4.12.4 -i https://pypi.org/simple/
 ############################################# 
 from setuptools import setup, find_packages,Extension
 import os
@@ -15,10 +15,10 @@ def file_get_content(k):
         con=''
     return con
 confkcw={}
-confkcw['name']='kcweb'                             #项目的名称
-confkcw['version']='3.0.0'							#项目版本
+confkcw['name']='kcweb'                             #项目的名称 
+confkcw['version']='4.13.9'							#项目版本
 confkcw['description']=''       #项目的简单描述
-confkcw['long_description']=""     #项目详细描述
+confkcw['long_description']="增加任务队列进度条"     #项目详细描述
 confkcw['license']='MIT License'                    #开源协议   mit开源
 confkcw['url']=''
 confkcw['author']='禄可集团-坤坤'  					 #名字
@@ -51,8 +51,8 @@ setup(
     maintainer_email = confkcw["maintainer_email"],
     url=confkcw['url'],
     packages =  b,
-    scripts = ['bin/kcw',],
-    install_requires = ['pymongo>=3.10.0','Mako>=1.1.2','requests>=2.23.0','six>=1.12.0','watchdog>=0.9.0'], #第三方包
+    # data_files=[('Scripts', ['kcweb/bin/kcw.exe'])],
+    install_requires = ['pymongo>=3.10.0','Mako>=1.1.2','requests>=2.23.0','six>=1.12.0','watchdog>=0.9.0','websockets==8.1'], #第三方包
     package_data = {
         '': ['*.html', '*.js','*.css','*.jpg','*.png','*.gif'],
     }
